/* David Leonard, 2003. Public domain. */
/* $Id: Card.c,v 1.2 2003/04/22 04:59:36 d Exp $ */

/*
 * A Card gadget.
 * These RectObjs are simply pixmap regions with an attached card ID.
 * The pixmaps are drawn by the Stack widget.
 */

#include <stdio.h>
#include <X11/IntrinsicP.h>
#include "CardP.h"

#ifdef DEBUG
#define HERE()	fprintf(stderr, "%s()\n", __FUNCTION__)
#else
#define HERE()	/* nothing */
#endif

static void CardClassInitialize(void);
static void CardClassPartInitialize(WidgetClass);
static void CardInitialize(Widget, Widget, ArgList, Cardinal*);
static void CardDestroy(Widget);

static XtResource resources[] = {
/*  { name, class, type,
      size, offset,
      default_type, default_addr }, */
    { XtNcardID, XtCCardID, XtRShort,
      sizeof(short), XtOffsetOf(CardRec, card.card_id),
      XtRImmediate, (XtPointer)CARD_NONE },
};

CardClassRec cardClassRec = {
    /* rectangle */
    {
	(WidgetClass)&rectObjClassRec,		/* superclass */
	"Card",					/* class_name */
	sizeof(CardRec),			/* widget_size */
	CardClassInitialize,			/* class_initialize */
	CardClassPartInitialize,		/* class_part_initialize */
	False,					/* class_initialized */
	CardInitialize,				/* initialize */
	NULL,					/* initialize_hook */
	NULL,					/* realize */
	NULL,					/* actions */
	0,					/* num_actions */
	resources,				/* resources */
	XtNumber(resources),			/* num_resources */
	NULLQUARK,				/* xrm_class */
	False,					/* compress_motion */
	False,					/* compress_exposure */
	False,					/* compress_enterleave */
	False,					/* visible_interest */
	CardDestroy,				/* destroy */
	XtInheritResize,			/* resize */
	XtInheritExpose,			/* expose */
	NULL,					/* set_values */
	NULL,					/* set_values_hook */
	XtInheritSetValuesAlmost,		/* set_values_almost */
	NULL,					/* get_values_hook */
	NULL,					/* accept_focus */
	XtVersion,				/* intrinsics_version */
	NULL,					/* callback offsets */
	NULL,					/* tm_table */
	XtInheritQueryGeometry,			/* query_geometry */
	NULL,					/* display_accelerator */
	NULL,					/* extension */
    },
    /* card */
    {
	NULL,					/* extension */
    }
};

WidgetClass cardObjectClass = (WidgetClass)&cardClassRec;

static void
CardClassInitialize()
{
	HERE();
}

static void
CardClassPartInitialize(WidgetClass cclass)
{
	HERE();
}

/*
 * Initialise a newly create gadgets
 */
static void
CardInitialize(request, cnew, args, num_args)
	Widget request, cnew;
	ArgList args;
	Cardinal *num_args;
{
	CardObject co = (CardObject)cnew;

	HERE();
	co->rectangle.border_width = 0;
}

static void
CardDestroy(Widget w)
{
	HERE();
}

#if 0
/* 
 * Compute our preferred geometry
 */
static XtGeometryResult
CardQueryGeometry(w, intended, return_val)
	Widget w;
	XtWidgetGeometry *intended;
	XtWidgetGeometry *return_val;
{
	XtGeometryResult ret = XtGeometryYes;

	HERE();
	if ((intended->request_mode & CWBorderWidth) && 
	    intended->border_width != 0)
	{
		return_val->request_mode |= CWBorderWidth;
		return_val->border_width = 0;
		ret = XtGeometryAlmost;
	}
	return ret;
}
#endif

/*------------------------------------------------------------
 * Utility functions
 */

#include <stdlib.h>
#include <time.h>

/*
 * Fill the array cards[CARD_STANDARD_DECK_SIZE] with unique cards
 */
void
CardFillStandardDeck(CardID *cards)
{
	CardID suits[] = { SUIT_DIAMONDS, SUIT_HEARTS,
			   SUIT_CLUBS, SUIT_SPADES };
	CardID faces[] = { FACE_ACE, 2, 3, 4, 5, 6, 7, 8, 9,
			   FACE_TEN, FACE_JACK, FACE_QUEEN, FACE_KING };
	int s, f;
	int pos;

	/*assert(XtNumber(suits)*XtNumber(faces)) == CARD_STANDARD_DECK_SIZE);*/

	pos = 0;
	for (s = 0; s < XtNumber(suits); s++)
	  for (f = 0; f < XtNumber(faces); f++)
	    cards[pos++] = suits[s] | faces[f];
	if (pos != CARD_STANDARD_DECK_SIZE)
	    XtError("CardFillStandardDeck: card miscount");
}

/*
 * Shuffle the array of cards
 */
void
CardShuffle(CardID *cards, Cardinal num_cards)
{
	int i;

	srand(time(0));
	for (i = 0; i < num_cards; i++) {
		CardID c;
		int j = rand() % num_cards;

		c = cards[i];
		cards[i] = cards[j];
		cards[j] = c;
	}
}

String
CardSuitName(CardID card)
{
	switch (CARD_SUIT(card)) {
	case SUIT_SPADES:	return "spades";
	case SUIT_DIAMONDS:	return "diamonds";
	case SUIT_CLUBS:	return "clubs";
	case SUIT_HEARTS:	return "hearts";
	}
	return "unknown";
}

String 
CardFaceName(CardID card)
{
	switch (CARD_FACE(card)) {
	case FACE_ACE:		return "ace";
	case 2:			return "2";
	case 3:			return "3";
	case 4:			return "4";
	case 5:			return "5";
	case 6:			return "6";
	case 7:			return "7";
	case 8:			return "8";
	case 9:			return "9";
	case FACE_TEN:		return "10";
	case FACE_JACK:		return "jack";
	case FACE_QUEEN:	return "queen";
	case FACE_KING: 	return "king";
	case FACE_JOKER:	return "joker";
	}
	return "unknown";
}
