/* David Leonard, 2003. Public domain. */
/* $Id: Card.h,v 1.2 2003/04/22 04:59:36 d Exp $ */

#ifndef _Card_h
#define _Card_h

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/RectObj.h>

/* Resources:

 Name                Class              RepType         Default Value
 ----                -----              -------         -------------
 cardID              CardID             Short           0 (CARD_NONE)

*/

#define XtNcardID        "cardID"
#define XtCCardID        "CardID"

typedef struct _CardClassRec *CardObjectClass;
typedef struct _CardRec *CardObject;
extern WidgetClass cardObjectClass;

typedef short CardID;

/*
 * Card IDs are encoded as thus:
 *    7      6      5      4      3      2      1       0
 * +------+------+------+------+------+------+------+------+
 * | down | spec | clr  | suit |        face value         |
 * +------+------+------+------+------+------+------+------+
 *   0x80   0x40   0x20   0x10
 */

#define CARD_SUIT(c)	((c) & SUIT_MASK)	/* card's suit */
#define CARD_FACE(c)	((c) & FACE_MASK)	/* card's face value */
#define CARD_COLOUR(c)	((c) & COLOUR_MASK)	/* card/suit colour */
#define CARD_IS_DOWN(c)	((c) & CARD_DOWN)	/* card is face-down */

#define CARD_MASK	(SUIT_MASK | FACE_MASK)	/* unique part of card */

#define CARD_DOWN	0x80

#define	CARD_NONE	0			/* "no card" */
#define	CARD_BLACK_JOKER  (SUIT_SPECIAL | COLOUR_BLACK | FACE_JOKER)
#define	CARD_RED_JOKER  (SUIT_SPECIAL | COLOUR_RED | FACE_JOKER)
#define	_CARD_PLACE	(SUIT_SPECIAL | 1)	/* DO NOT USE */
#define	_CARD_BACK	(SUIT_SPECIAL | 2)	/* DO NOT USE */

#define COLOUR_BLACK	0x20
#define COLOUR_RED	0x00
#define COLOUR_MASK	0x20		/* colour of the card */

#define FACE_ACE	0x01		/* Will always be values 1..13 */
#define FACE_TEN	0x0A
#define FACE_JACK	0x0B
#define FACE_QUEEN	0x0C
#define FACE_KING	0x0D
#define FACE_JOKER	0x0E		/* (joker value may change) */
#define FACE_MASK	0x0F		/* the face value of the card */

#define SUIT_SPECIAL	0x00
#define SUIT_NORMAL	0x40
#define SUIT_MASK	(COLOUR_MASK  | 0x10 | SUIT_NORMAL)
#define SUIT_DIAMONDS	(COLOUR_RED   | 0x00 | SUIT_NORMAL)
#define SUIT_HEARTS	(COLOUR_RED   | 0x10 | SUIT_NORMAL)
#define SUIT_CLUBS	(COLOUR_BLACK | 0x00 | SUIT_NORMAL)
#define SUIT_SPADES	(COLOUR_BLACK | 0x10 | SUIT_NORMAL)

#define CARD_STANDARD_DECK_SIZE (13 * 4)

extern void CardFillStandardDeck(
#if NeedFunctionPrototypes
	CardID *	/* cards */
#endif
);

extern void CardShuffle(
#if NeedFunctionPrototypes
	CardID *	/* cards */,
	Cardinal	/* num_cards */
#endif
);

extern String CardSuitName(
#if NeedFunctionPrototypes
	CardID		/* card */
#endif
);

extern String CardFaceName(
#if NeedFunctionPrototypes
	CardID		/* card */
#endif
);

#endif _Card_h
