/* David Leonard, 2003. Public domain. */
/* $Id: Stack.h,v 1.2 2003/04/22 04:59:36 d Exp $ */

#ifndef _Stack_h
#define _Stack_h

#include <X11/Xmu/Converters.h>
#include <X11/StringDefs.h>
#include "Card.h"

/*
 * Stack widget
 */

/* Resources:

 Name			Class		RepType		Default Value
 ----			-----		-------		-------------
 foreground		Foreground	Pixel		XtDefaultForeground
 dropCallback		Callback	Pointer		StackAlwaysDrop
 splitCallback		Callback	Pointer		StackAlwaysSplit
 dropNotify		Callback	XtCallbackList	NULL
 hitNotify		Callback	XtCallbackList	NULL
 compact		Compact		Boolean		False
 highlight     		Foreground	Pixel		XtDefaultForeground
 highlightWidth		Width		Dimension	1
 highlightHeight	Height		Dimension	1
 gravity		Gravity		XtGravity	Center
 vStepMin		VStepMin	Dimension	8
 vStepMax		VStepMax	Dimension	24
 hStepMin		HStepMin	Dimension	8
 hStepMax		HStepMax	Dimension	16
 placeholderPixmap	CardPixmap	Pixmap		None
 aceDiamondsPixmap	CardPixmap	Pixmap		None
 twoDiamondsPixmap	CardPixmap	Pixmap		None
 threeDiamondsPixmap	CardPixmap	Pixmap		None
 fourDiamondsPixmap	CardPixmap	Pixmap		None
 fiveDiamondsPixmap	CardPixmap	Pixmap		None
 sixDiamondsPixmap	CardPixmap	Pixmap		None
 sevenDiamondsPixmap	CardPixmap	Pixmap		None
 eightDiamondsPixmap	CardPixmap	Pixmap		None
 nineDiamondsPixmap	CardPixmap	Pixmap		None
 tenDiamondsPixmap	CardPixmap	Pixmap		None
 jackDiamondsPixmap	CardPixmap	Pixmap		None
 queenDiamondsPixmap	CardPixmap	Pixmap		None
 kingDiamondsPixmap	CardPixmap	Pixmap		None
 aceHeartsPixmap	CardPixmap	Pixmap		None
 twoHeartsPixmap	CardPixmap	Pixmap		None
 threeHeartsPixmap	CardPixmap	Pixmap		None
 fourHeartsPixmap	CardPixmap	Pixmap		None
 fiveHeartsPixmap	CardPixmap	Pixmap		None
 sixHeartsPixmap	CardPixmap	Pixmap		None
 sevenHeartsPixmap	CardPixmap	Pixmap		None
 eightHeartsPixmap	CardPixmap	Pixmap		None
 nineHeartsPixmap	CardPixmap	Pixmap		None
 tenHeartsPixmap	CardPixmap	Pixmap		None
 jackHeartsPixmap	CardPixmap	Pixmap		None
 queenHeartsPixmap	CardPixmap	Pixmap		None
 kingHeartsPixmap	CardPixmap	Pixmap		None
 aceClubsPixmap		CardPixmap	Pixmap		None
 twoClubsPixmap		CardPixmap	Pixmap		None
 threeClubsPixmap	CardPixmap	Pixmap		None
 fourClubsPixmap	CardPixmap	Pixmap		None
 fiveClubsPixmap	CardPixmap	Pixmap		None
 sixClubsPixmap		CardPixmap	Pixmap		None
 sevenClubsPixmap	CardPixmap	Pixmap		None
 eightClubsPixmap	CardPixmap	Pixmap		None
 nineClubsPixmap	CardPixmap	Pixmap		None
 tenClubsPixmap		CardPixmap	Pixmap		None
 jackClubsPixmap	CardPixmap	Pixmap		None
 queenClubsPixmap	CardPixmap	Pixmap		None
 kingClubsPixmap	CardPixmap	Pixmap		None
 aceSpadesPixmap	CardPixmap	Pixmap		None
 twoSpadesPixmap	CardPixmap	Pixmap		None
 threeSpadesPixmap	CardPixmap	Pixmap		None
 fourSpadesPixmap	CardPixmap	Pixmap		None
 fiveSpadesPixmap	CardPixmap	Pixmap		None
 sixSpadesPixmap	CardPixmap	Pixmap		None
 sevenSpadesPixmap	CardPixmap	Pixmap		None
 eightSpadesPixmap	CardPixmap	Pixmap		None
 nineSpadesPixmap	CardPixmap	Pixmap		None
 tenSpadesPixmap	CardPixmap	Pixmap		None
 jackSpadesPixmap	CardPixmap	Pixmap		None
 queenSpadesPixmap	CardPixmap	Pixmap		None
 kingSpadesPixmap	CardPixmap	Pixmap		None
 jokerRedPixmap		CardPixmap	Pixmap		None
 jokerBlackPixmap	CardPixmap	Pixmap		None
 backPixmap		CardPixmap	Pixmap		None
 
Allowable values for the gravity resource are:
	South
	East
	West
	North
	Center

*/

/* define any special resource names here that are not in <X11/StringDefs.h> */
#define XtNdropCallback  "dropCallback"
#define XtNsplitCallback "splitCallback"
#define XtNhitNotify     "hitNotify"
#define XtNdropNotify    "dropNotify"
#define XtNgravity	 "gravity"
#define XtNvStepMin	 "vStepMin"
#define XtNvStepMax	 "vStepMax"
#define XtNhStepMin	 "hStepMin"
#define XtNhStepMax	 "hStepMax"
#define XtNcompact	 "compact"
#define XtNhighlightWidth "highlightWidth"
#define XtNhighlightHeight "highlightHeight"

#define XtNplaceholderPixmap	"placeholderPixmap"
#define XtNaceDiamondsPixmap	"aceDiamondsPixmap"
#define XtNtwoDiamondsPixmap	"twoDiamondsPixmap"
#define XtNthreeDiamondsPixmap	"threeDiamondsPixmap"
#define XtNfourDiamondsPixmap	"fourDiamondsPixmap"
#define XtNfiveDiamondsPixmap	"fiveDiamondsPixmap"
#define XtNsixDiamondsPixmap	"sixDiamondsPixmap"
#define XtNsevenDiamondsPixmap	"sevenDiamondsPixmap"
#define XtNeightDiamondsPixmap	"eightDiamondsPixmap"
#define XtNnineDiamondsPixmap	"nineDiamondsPixmap"
#define XtNtenDiamondsPixmap	"tenDiamondsPixmap"
#define XtNjackDiamondsPixmap	"jackDiamondsPixmap"
#define XtNqueenDiamondsPixmap	"queenDiamondsPixmap"
#define XtNkingDiamondsPixmap	"kingDiamondsPixmap"
#define XtNaceHeartsPixmap	"aceHeartsPixmap"
#define XtNtwoHeartsPixmap	"twoHeartsPixmap"
#define XtNthreeHeartsPixmap	"threeHeartsPixmap"
#define XtNfourHeartsPixmap	"fourHeartsPixmap"
#define XtNfiveHeartsPixmap	"fiveHeartsPixmap"
#define XtNsixHeartsPixmap	"sixHeartsPixmap"
#define XtNsevenHeartsPixmap	"sevenHeartsPixmap"
#define XtNeightHeartsPixmap	"eightHeartsPixmap"
#define XtNnineHeartsPixmap	"nineHeartsPixmap"
#define XtNtenHeartsPixmap	"tenHeartsPixmap"
#define XtNjackHeartsPixmap	"jackHeartsPixmap"
#define XtNqueenHeartsPixmap	"queenHeartsPixmap"
#define XtNkingHeartsPixmap	"kingHeartsPixmap"
#define XtNaceClubsPixmap	"aceClubsPixmap"
#define XtNtwoClubsPixmap	"twoClubsPixmap"
#define XtNthreeClubsPixmap	"threeClubsPixmap"
#define XtNfourClubsPixmap	"fourClubsPixmap"
#define XtNfiveClubsPixmap	"fiveClubsPixmap"
#define XtNsixClubsPixmap	"sixClubsPixmap"
#define XtNsevenClubsPixmap	"sevenClubsPixmap"
#define XtNeightClubsPixmap	"eightClubsPixmap"
#define XtNnineClubsPixmap	"nineClubsPixmap"
#define XtNtenClubsPixmap	"tenClubsPixmap"
#define XtNjackClubsPixmap	"jackClubsPixmap"
#define XtNqueenClubsPixmap	"queenClubsPixmap"
#define XtNkingClubsPixmap	"kingClubsPixmap"
#define XtNaceSpadesPixmap	"aceSpadesPixmap"
#define XtNtwoSpadesPixmap	"twoSpadesPixmap"
#define XtNthreeSpadesPixmap	"threeSpadesPixmap"
#define XtNfourSpadesPixmap	"fourSpadesPixmap"
#define XtNfiveSpadesPixmap	"fiveSpadesPixmap"
#define XtNsixSpadesPixmap	"sixSpadesPixmap"
#define XtNsevenSpadesPixmap	"sevenSpadesPixmap"
#define XtNeightSpadesPixmap	"eightSpadesPixmap"
#define XtNnineSpadesPixmap	"nineSpadesPixmap"
#define XtNtenSpadesPixmap	"tenSpadesPixmap"
#define XtNjackSpadesPixmap	"jackSpadesPixmap"
#define XtNqueenSpadesPixmap	"queenSpadesPixmap"
#define XtNkingSpadesPixmap	"kingSpadesPixmap"
#define XtNjokerRedPixmap	"jokerRedPixmap"
#define XtNjokerBlackPixmap	"jokerBlackPixmap"
#define XtNbackPixmap		"backPixmap"

#define XtCGravity	 "Gravity"
#define XtCVStepMin	 "VStepMin"
#define XtCVStepMax	 "VStepMax"
#define XtCHStepMin	 "HStepMin"
#define XtCHStepMax	 "HStepMax"
#define XtCCardPixmap	 "CardPixmap"
#define XtCCompact	 "Compact"

typedef struct _StackClassRec *StackWidgetClass;
typedef struct _StackRec *StackWidget;

extern WidgetClass stackWidgetClass;

typedef struct _StackDropEvent {
	Widget		source;
	Widget		target;
	Cardinal	count;
} StackDropEvent;

typedef Boolean	(*StackSplitCallbackProc)(
#if NeedFunctionPrototypes
	Widget		/* source */,
	XtPointer 	/* closure */,			/* always NULL */
	Cardinal 	/* depth */
#endif
);

#define StackNeverSplit		&StackNeverSplitProc
#define StackAlwaysSplit	(StackSplitCallbackProc)NULL

typedef Boolean	(*StackDropCallbackProc)(
#if NeedFunctionPrototypes
	Widget		/* target */,
	XtPointer	/* closure */,			/* always NULL */
	CardID *	/* cards */,
	Cardinal	/* num_cards */,
	Widget		/* source */
#endif
);

#define StackAlwaysDrop		(StackDropCallbackProc)NULL
#define StackNeverDrop		&StackNeverDropProc

extern Cardinal	StackDepth(
#if NeedFunctionPrototypes
	Widget		/* stack */
#endif
);

extern CardID StackPeek(
#if NeedFunctionPrototypes
	Widget 		/* stack */,
	Cardinal 	/* depth */
#endif
);

extern void StackPush(
#if NeedFunctionPrototypes
	Widget 		/* stack */,
	CardID 		/* card */
#endif
);

extern CardID StackPop(
#if NeedFunctionPrototypes
	Widget 		/* stack */
#endif
);

extern void StackClear(
#if NeedFunctionPrototypes
	Widget 		/* stack */
#endif
);

extern void StackMove(
#if NeedFunctionPrototypes
	Widget		/* dst */,
	Widget		/* src */,
	Cardinal	/* count */
#endif
);

extern Widget StackCreateDragAndDropPopup(
#if NeedFunctionPrototypes
	Widget		/* toplevel */,
	String		/* popup_name */
#endif
);

#define	StackTop(stack)	StackPeek(stack, 0)

extern Boolean StackNeverSplitProc(
#if NeedFunctionPrototypes
	Widget		/* source */,
	XtPointer 	/* closure */,			/* always NULL */
	Cardinal 	/* depth */,
	Boolean		/* check_only */
#endif
);

extern Boolean StackNeverDropProc(
#if NeedFunctionPrototypes
	Widget		/* target */,
	XtPointer	/* closure */,			/* always NULL */
	CardID *	/* cards */,
	Cardinal	/* num_cards */,
	Widget		/* source */,
	Boolean		/* check_only */
#endif
);

#endif
